from __future__ import annotations

from html.parser import HTMLParser


class AnchorParser(HTMLParser):
    def __init__(self):
        super().__init__()
        self.anchors = []

    def handle_starttag(self, tag, attrs):
        if tag == "a":
            self.anchors.append(attrs[0][1])


def count_anchors(html: str) -> int:
    parser = AnchorParser()
    parser.feed(html)
    return len(parser.anchors)



from __future__ import annotations

import pytest
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.utils.module_loading import import_string
from model_bakery import baker

from django_simple_nav.nav import NavGroup
from django_simple_nav.nav import NavItem
from django_simple_nav.nav import RenderedNavItem
from tests.navs import DummyNav
from tests.utils import count_anchors

pytestmark = pytest.mark.django_db


def test_init():
    nav = DummyNav

    assert nav.template_name == "tests/dummy_nav.html"
    assert len(nav.items) == 12

    for item in nav.items:
        assert item.title


@pytest.mark.parametrize(
    "nav, template_name, expected_count",
    [
        ("tests.navs.DummyNav", "tests/dummy_nav.html", 12),
    ],
)
def test_dotted_path_loading(nav, template_name, expected_count):
    nav = import_string(nav)

    assert nav.template_name == template_name
    assert len(nav.items) == expected_count


@pytest.mark.parametrize(
    "user, expected_count",
    [
        (AnonymousUser(), 7),
        (None, 10),
    ],
)
def test_nav_render(user, expected_count, req):
    if not isinstance(user, AnonymousUser):
        user = baker.make(get_user_model())

    req.user = user

    rendered_template = DummyNav().render(req)

    assert count_anchors(rendered_template) == expected_count


def test_dotted_path_nav_render(req):
    req.user = baker.make(get_user_model())

    nav = import_string("tests.navs.DummyNav")

    assert nav().render(req)


@pytest.mark.parametrize(
    "permission, expected_count",
    [
        ("", 10),  # regular authenticated user
        ("is_staff", 13),
        ("is_superuser", 19),
        ("tests.dummy_perm", 13),
    ],
)
def test_nav_render_with_permissions(req, permission, expected_count):
    user = baker.make(get_user_model())

    if permission == "tests.dummy_perm":
        dummy_perm = baker.make(
            "auth.Permission",
            codename="dummy_perm",
            name="Dummy Permission",
            content_type=baker.make("contenttypes.ContentType", app_label="tests"),
        )
        user.user_permissions.add(dummy_perm)
    else:
        setattr(user, permission, True)

    user.save()

    req.user = user
    rendered_template = DummyNav().render(req)

    assert count_anchors(rendered_template) == expected_count


def test_nav_render_with_template_name(req):
    req.user = AnonymousUser()

    rendered_template = DummyNav().render(req, "tests/alternate.html")

    assert "This is an alternate template." in rendered_template


def test_extra_context(req):
    item = NavItem(
        title="Test",
        url="/test/",
        extra_context={"foo": "bar"},
    )

    rendered_item = RenderedNavItem(item, req)

    assert rendered_item.foo == "bar"


def test_extra_context_with_no_extra_context(req):
    item = NavItem(
        title="Test",
        url="/test/",
    )

    rendered_item = RenderedNavItem(item, req)

    with pytest.raises(AttributeError):
        assert rendered_item.foo == "bar"


def test_extra_context_shadowing(req):
    item = NavItem(
        title="Test",
        url="/test/",
        extra_context={"title": "Shadowed"},
    )

    rendered_item = RenderedNavItem(item, req)

    assert rendered_item.title == "Test"


def test_extra_context_iteration(req):
    item = NavItem(
        title="Test",
        url="/test/",
        extra_context={"foo": "bar", "baz": "qux"},
    )

    rendered_item = RenderedNavItem(item, req)

    assert rendered_item.extra_context == {"foo": "bar", "baz": "qux"}
    for key, value in rendered_item.extra_context.items():
        assert getattr(rendered_item, key) == value


def test_extra_context_builtins(req):
    item = NavGroup(
        title="Test",
        items=[
            NavItem(
                title="Test",
                url="/test/",
                permissions=["is_staff"],
                extra_context={"foo": "bar"},
            ),
        ],
        url="/test/",
        permissions=["is_staff"],
        extra_context={"baz": "qux"},
    )

    rendered_item = RenderedNavItem(item, req)

    assert rendered_item.title == "Test"
    assert rendered_item.url == "/test/"
    assert rendered_item.permissions == ["is_staff"]
    assert rendered_item.extra_context == {"baz": "qux"}
    assert rendered_item.baz == "qux"

    assert rendered_item.items is not None
    assert len(rendered_item.items) == 1

    rendered_group_item = rendered_item.items[0]

    assert rendered_group_item.title == "Test"
    assert rendered_group_item.url == "/test/"
    assert rendered_group_item.permissions == ["is_staff"]
    assert rendered_group_item.extra_context == {"foo": "bar"}
    assert rendered_group_item.foo == "bar"


def test_get_context_data(req):
    req.user = baker.make(get_user_model())

    context = DummyNav().get_context_data(req)

    assert context["items"]


def test_get_context_data_override(req):
    class OverrideNav(DummyNav):
        def get_context_data(self, request):
            return {"foo": "bar"}

    req.user = baker.make(get_user_model())

    context = OverrideNav().get_context_data(req)

    assert context["foo"] == "bar"


def test_get_context_data_override_render(req):
    class OverrideNav(DummyNav):
        def get_context_data(self, request):
            return {"foo": "bar"}

    req.user = baker.make(get_user_model())

    rendered_template = OverrideNav().render(req)

    assert count_anchors(rendered_template) == 0


def test_rendered_nav_item_active(req):
    item = NavItem(title="Test", url="/test/")
    rendered_item = RenderedNavItem(item, req)

    assert rendered_item.active is False

    req.path = "/test/"
    rendered_item = RenderedNavItem(item, req)

    assert rendered_item.active is True


def test_rendered_nav_group_active_no_url(req):
    item = NavGroup(title="Test", items=[NavItem(title="Test", url="/test/")])

    rendered_item = RenderedNavItem(item, req)

    assert rendered_item.active is False


def test_rendered_nav_item_active_named_url(req):
    item = NavItem(title="Test", url="fake-view")

    req.path = "/fake-view/"
    rendered_item = RenderedNavItem(item, req)

    assert rendered_item.active is True



from __future__ import annotations

import logging

import pytest
from django.conf import settings
from django.http import HttpRequest

from .settings import DEFAULT_SETTINGS

pytest_plugins = []  # type: ignore


def pytest_configure(config):
    logging.disable(logging.CRITICAL)

    settings.configure(**DEFAULT_SETTINGS, **TEST_SETTINGS)


TEST_SETTINGS = {
    "INSTALLED_APPS": [
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django_simple_nav",
        "tests",
    ],
    "ROOT_URLCONF": "tests.urls",
    "TEMPLATES": [
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "APP_DIRS": True,
        }
    ],
}


@pytest.fixture
def req():
    return HttpRequest()



from __future__ import annotations

import logging
from typing import TYPE_CHECKING
from typing import Protocol
from typing import cast

from django.apps import apps
from django.http import HttpRequest

if TYPE_CHECKING:
    from django_simple_nav.nav import NavGroup
    from django_simple_nav.nav import NavItem

logger = logging.getLogger(__name__)


class User(Protocol):
    is_authenticated: bool
    is_staff: bool
    is_superuser: bool

    def has_perm(self, perm: str) -> bool: ...  # pragma: no cover


def check_item_permissions(item: NavGroup | NavItem, request: HttpRequest) -> bool:
    if not apps.is_installed("django.contrib.auth"):
        logger.warning(
            "The 'django.contrib.auth' app is not installed, so no permissions will be checked."
        )
        return True

    user = cast(User, request.user)

    for idx, perm in enumerate(item.permissions):
        if not hasattr(request, "user"):
            return False

        user = cast(User, request.user)

        user_perm = user_has_perm(user, perm)

        if not user_perm:
            return False

        if not idx == len(item.permissions) - 1:
            continue

    if hasattr(item, "items"):
        sub_items = [
            sub_item
            for sub_item in item.items
            if check_item_permissions(sub_item, request)
        ]
        if not sub_items and not item.url:
            return False

    return True


def user_has_perm(user: User, perm: str) -> bool:
    """Check if the user has a certain auth attribute or permission."""

    has_perm = False

    if perm in ["is_authenticated", "is_staff", "is_superuser"]:
        is_superuser = getattr(user, "is_superuser", False)
        if is_superuser:
            has_perm = True
        else:
            has_perm = getattr(user, perm, False)
    elif user.has_perm(perm):
        has_perm = True

    return has_perm



from __future__ import annotations

import pytest
from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.test import override_settings
from model_bakery import baker

from django_simple_nav.nav import NavGroup
from django_simple_nav.nav import NavItem
from django_simple_nav.permissions import check_item_permissions
from django_simple_nav.permissions import user_has_perm

pytestmark = pytest.mark.django_db


def test_check_anonymous_user_has_perm():
    user = AnonymousUser()

    assert not user_has_perm(user, "is_authenticated")


def test_check_authenticated_user_has_perm():
    user = baker.make(get_user_model())

    assert user_has_perm(user, "is_authenticated")


def test_check_staff_user_has_perm():
    user = baker.make(get_user_model(), is_staff=True)

    assert user_has_perm(user, "is_staff")


def test_check_superuser_user_has_perm():
    user = baker.make(get_user_model(), is_superuser=True)

    assert user_has_perm(user, "is_superuser")


def test_check_auth_permission_user_has_perm():
    user = baker.make(get_user_model())

    dummy_perm = baker.make(
        "auth.Permission",
        codename="dummy_perm",
        name="Dummy Permission",
        content_type=baker.make("contenttypes.ContentType", app_label="tests"),
    )

    user.user_permissions.add(dummy_perm)

    assert user_has_perm(user, "tests.dummy_perm")


# anonymous user
@pytest.mark.parametrize(
    "item,expected",
    [
        # nav item
        (NavItem("Test", "/test"), True),
        (NavItem("Test", "/test", permissions=["is_authenticated"]), False),
        (NavItem("Test", "/test", permissions=["is_staff"]), False),
        (NavItem("Test", "/test", permissions=["is_superuser"]), False),
        (NavItem("Test", "/test", permissions=["tests.dummy_perm"]), False),
        # nav group with url
        (NavGroup("Test", [], "/test"), True),
        (NavGroup("Test", [], "/test", permissions=["is_authenticated"]), False),
        (NavGroup("Test", [], "/test", permissions=["is_staff"]), False),
        (NavGroup("Test", [], "/test", permissions=["is_superuser"]), False),
        (NavGroup("Test", [], "/test", permissions=["tests.dummy_perm"]), False),
        # nav group with no url and items with perms
        (NavGroup("Test", [NavItem("Test", "/test")]), True),
        (
            NavGroup(
                "Test",
                [NavItem("Test", "/test", permissions=["is_authenticated"])],
            ),
            False,
        ),
        (NavGroup("Test", [NavItem("Test", "/test", permissions=["is_staff"])]), False),
        (
            NavGroup("Test", [NavItem("Test", "/test", permissions=["is_superuser"])]),
            False,
        ),
        (
            NavGroup(
                "Test", [NavItem("Test", "/test", permissions=["tests.dummy_perm"])]
            ),
            False,
        ),
        # multiple perms
        (NavItem("Test", "/test", permissions=["is_authenticated", "is_staff"]), False),
        (
            NavItem(
                "Test",
                "/test",
                permissions=["is_authenticated", "is_staff", "is_superuser"],
            ),
            False,
        ),
    ],
)
def test_check_item_permissions_anonymous(item, expected, req):
    req.user = AnonymousUser()

    assert check_item_permissions(item, req) == expected


# authenticated user
@pytest.mark.parametrize(
    "item,expected",
    [
        # nav item
        (NavItem("Test", "/test"), True),
        (NavItem("Test", "/test", permissions=["is_authenticated"]), True),
        (NavItem("Test", "/test", permissions=["is_staff"]), False),
        (NavItem("Test", "/test", permissions=["is_superuser"]), False),
        (NavItem("Test", "/test", permissions=["tests.dummy_perm"]), False),
        # nav group with url
        (NavGroup("Test", [], "/test"), True),
        (NavGroup("Test", [], "/test", permissions=["is_authenticated"]), True),
        (NavGroup("Test", [], "/test", permissions=["is_staff"]), False),
        (NavGroup("Test", [], "/test", permissions=["is_superuser"]), False),
        (NavGroup("Test", [], "/test", permissions=["tests.dummy_perm"]), False),
        # nav group with no url and items with perms
        (NavGroup("Test", [NavItem("Test", "/test")], "/test"), True),
        (
            NavGroup(
                "Test",
                [NavItem("Test", "/test", permissions=["is_authenticated"])],
            ),
            True,
        ),
        (NavGroup("Test", [NavItem("Test", "/test", permissions=["is_staff"])]), False),
        (
            NavGroup("Test", [NavItem("Test", "/test", permissions=["is_superuser"])]),
            False,
        ),
        (
            NavGroup(
                "Test", [NavItem("Test", "/test", permissions=["tests.dummy_perm"])]
            ),
            False,
        ),
        # multiple perms
        (NavItem("Test", "/test", permissions=["is_authenticated", "is_staff"]), False),
        (
            NavItem(
                "Test",
                "/test",
                permissions=["is_authenticated", "is_staff", "is_superuser"],
            ),
            False,
        ),
    ],
)
def test_check_item_permissions_is_authenticated(item, expected, req):
    req.user = baker.make(get_user_model())

    assert check_item_permissions(item, req) == expected


# staff user
@pytest.mark.parametrize(
    "item,expected",
    [
        # nav item
        (NavItem("Test", "/test"), True),
        (NavItem("Test", "/test", permissions=["is_authenticated"]), True),
        (NavItem("Test", "/test", permissions=["is_staff"]), True),
        (NavItem("Test", "/test", permissions=["is_superuser"]), False),
        (NavItem("Test", "/test", permissions=["tests.dummy_perm"]), False),
        # nav group with url
        (NavGroup("Test", [], "/test"), True),
        (NavGroup("Test", [], "/test", permissions=["is_authenticated"]), True),
        (NavGroup("Test", [], "/test", permissions=["is_staff"]), True),
        (NavGroup("Test", [], "/test", permissions=["is_superuser"]), False),
        (NavGroup("Test", [], "/test", permissions=["tests.dummy_perm"]), False),
        # nav group with no url and items with perms
        (NavGroup("Test", [NavItem("Test", "/test")]), True),
        (
            NavGroup(
                "Test",
                [NavItem("Test", "/test", permissions=["is_authenticated"])],
            ),
            True,
        ),
        (NavGroup("Test", [NavItem("Test", "/test", permissions=["is_staff"])]), True),
        (
            NavGroup("Test", [NavItem("Test", "/test", permissions=["is_superuser"])]),
            False,
        ),
        (
            NavGroup(
                "Test", [NavItem("Test", "/test", permissions=["tests.dummy_perm"])]
            ),
            False,
        ),
        # multiple perms
        (NavItem("Test", "/test", permissions=["is_authenticated", "is_staff"]), True),
        (
            NavItem(
                "Test",
                "/test",
                permissions=["is_authenticated", "is_staff", "is_superuser"],
            ),
            False,
        ),
    ],
)
def test_check_item_permissions_is_staff(item, expected, req):
    req.user = baker.make(get_user_model(), is_staff=True)

    assert check_item_permissions(item, req) == expected


# superuser
@pytest.mark.parametrize(
    "item,expected",
    [
        # nav item
        (NavItem("Test", "/test"), True),
        (NavItem("Test", "/test", permissions=["is_authenticated"]), True),
        (NavItem("Test", "/test", permissions=["is_staff"]), True),
        (NavItem("Test", "/test", permissions=["is_superuser"]), True),
        (NavItem("Test", "/test", permissions=["tests.dummy_perm"]), True),
        # nav group with url
        (NavGroup("Test", [], "/test"), True),
        (NavGroup("Test", [], "/test", permissions=["is_authenticated"]), True),
        (NavGroup("Test", [], "/test", permissions=["is_staff"]), True),
        (NavGroup("Test", [], "/test", permissions=["is_superuser"]), True),
        (NavGroup("Test", [], "/test", permissions=["tests.dummy_perm"]), True),
        # nav group with no url and items with perms
        (NavGroup("Test", [NavItem("Test", "/test")], "/test"), True),
        (
            NavGroup(
                "Test",
                [NavItem("Test", "/test", permissions=["is_authenticated"])],
            ),
            True,
        ),
        (NavGroup("Test", [NavItem("Test", "/test", permissions=["is_staff"])]), True),
        (
            NavGroup("Test", [NavItem("Test", "/test", permissions=["is_superuser"])]),
            True,
        ),
        (
            NavGroup(
                "Test", [NavItem("Test", "/test", permissions=["tests.dummy_perm"])]
            ),
            True,
        ),
        # multiple perms
        (NavItem("Test", "/test", permissions=["is_authenticated", "is_staff"]), True),
        (
            NavItem(
                "Test",
                "/test",
                permissions=["is_authenticated", "is_staff", "is_superuser"],
            ),
            True,
        ),
    ],
)
def test_check_item_permissions_is_superuser(item, expected, req):
    req.user = baker.make(get_user_model(), is_superuser=True)

    assert check_item_permissions(item, req) == expected


# user with specific auth.Permission
@pytest.mark.parametrize(
    "item,expected",
    [
        # nav item
        (NavItem("Test", "/test"), True),
        (NavItem("Test", "/test", permissions=["is_authenticated"]), True),
        (NavItem("Test", "/test", permissions=["is_staff"]), False),
        (NavItem("Test", "/test", permissions=["is_superuser"]), False),
        (NavItem("Test", "/test", permissions=["tests.dummy_perm"]), True),
        # nav group with url
        (NavGroup("Test", [], "/test"), True),
        (NavGroup("Test", [], "/test", permissions=["is_authenticated"]), True),
        (NavGroup("Test", [], "/test", permissions=["is_staff"]), False),
        (NavGroup("Test", [], "/test", permissions=["is_superuser"]), False),
        (NavGroup("Test", [], "/test", permissions=["tests.dummy_perm"]), True),
        # nav group with no url and items with perms
        (NavGroup("Test", [NavItem("Test", "/test")], "/test"), True),
        (
            NavGroup(
                "Test",
                [NavItem("Test", "/test", permissions=["is_authenticated"])],
            ),
            True,
        ),
        (NavGroup("Test", [NavItem("Test", "/test", permissions=["is_staff"])]), False),
        (
            NavGroup("Test", [NavItem("Test", "/test", permissions=["is_superuser"])]),
            False,
        ),
        (
            NavGroup(
                "Test", [NavItem("Test", "/test", permissions=["tests.dummy_perm"])]
            ),
            True,
        ),
        # multiple perms
        (NavItem("Test", "/test", permissions=["is_authenticated", "is_staff"]), False),
        (
            NavItem(
                "Test",
                "/test",
                permissions=["is_authenticated", "is_staff", "is_superuser"],
            ),
            False,
        ),
    ],
)
def test_check_item_permissions_auth_permission(item, expected, req):
    user = baker.make(get_user_model())

    dummy_perm = baker.make(
        "auth.Permission",
        codename="dummy_perm",
        name="Dummy Permission",
        content_type=baker.make("contenttypes.ContentType", app_label="tests"),
    )

    user.user_permissions.add(dummy_perm)

    req.user = user

    assert check_item_permissions(item, req) == expected


@override_settings(
    INSTALLED_APPS=[
        app for app in settings.INSTALLED_APPS if app != "django.contrib.auth"
    ]
)
def test_check_item_permissions_no_contrib_auth(req, caplog):
    item = NavItem("Test", "/test", permissions=["is_authenticated"])

    with caplog.at_level("WARNING"):
        assert check_item_permissions(item, req) is True

    assert "The 'django.contrib.auth' app is not installed" in caplog.text



from __future__ import annotations

import pytest
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.template import Context
from django.template import Template
from django.template import TemplateSyntaxError
from model_bakery import baker

from django_simple_nav.nav import NavItem
from tests.navs import DummyNav
from tests.utils import count_anchors

pytestmark = pytest.mark.django_db


def test_django_simple_nav_templatetag(req):
    template = Template(
        "{% load django_simple_nav %} {% django_simple_nav 'tests.navs.DummyNav' %}"
    )
    req.user = AnonymousUser()

    rendered_template = template.render(Context({"request": req}))

    assert count_anchors(rendered_template) == 7


def test_templatetag_with_template_name(req):
    template = Template(
        "{% load django_simple_nav %} {% django_simple_nav 'tests.navs.DummyNav' 'tests/alternate.html' %}"
    )
    req.user = AnonymousUser()

    rendered_template = template.render(Context({"request": req}))

    assert "This is an alternate template." in rendered_template


def test_templatetag_with_nav_instance(req):
    class PlainviewNav(DummyNav):
        items = [
            NavItem(title="I drink your milkshake!", url="/milkshake/"),
        ]

    template = Template("{% load django_simple_nav %} {% django_simple_nav new_nav %}")
    req.user = baker.make(get_user_model(), first_name="Daniel", last_name="Plainview")

    rendered_template = template.render(
        Context({"request": req, "new_nav": PlainviewNav()})
    )

    assert "I drink your milkshake!" in rendered_template


def test_templatetag_with_nav_instance_and_template_name(req):
    class DeadParrotNav(DummyNav):
        items = [
            NavItem(title="He's pinin' for the fjords!", url="/notlob/"),
        ]

    template = Template(
        "{% load django_simple_nav %} {% django_simple_nav new_nav 'tests/alternate.html' %}"
    )
    req.user = baker.make(get_user_model(), first_name="Norwegian", last_name="Blue")

    rendered_template = template.render(
        Context({"request": req, "new_nav": DeadParrotNav()})
    )

    assert "He's pinin' for the fjords!" in rendered_template
    assert "This is an alternate template." in rendered_template


def test_templatetag_with_template_name_on_nav_instance(req):
    class PinkmanNav(DummyNav):
        template_name = "tests/alternate.html"
        items = [
            NavItem(title="Yeah Mr. White! Yeah science!", url="/science/"),
        ]

    template = Template("{% load django_simple_nav %} {% django_simple_nav new_nav %}")
    req.user = baker.make(get_user_model(), first_name="Jesse", last_name="Pinkman")

    rendered_template = template.render(
        Context({"request": req, "new_nav": PinkmanNav()})
    )

    assert "Yeah Mr. White! Yeah science!" in rendered_template
    assert "This is an alternate template." in rendered_template


def test_templatetag_with_no_arguments():
    with pytest.raises(TemplateSyntaxError):
        Template("{% load django_simple_nav %} {% django_simple_nav %}")


def test_templatetag_with_missing_variable():
    template = Template(
        "{% load django_simple_nav %} {% django_simple_nav missing_nav %}"
    )

    with pytest.raises(TemplateSyntaxError):
        template.render(Context({}))


def test_nested_templatetag(req):
    # called twice to simulate a nested call
    template = Template(
        "{% load django_simple_nav %} {% django_simple_nav 'tests.navs.DummyNav' %}"
        "{% django_simple_nav 'tests.navs.DummyNav' %}"
    )
    req.user = AnonymousUser()

    rendered_template = template.render(Context({"request": req}))

    assert count_anchors(rendered_template) == 14



from __future__ import annotations

from django_simple_nav.nav import Nav
from django_simple_nav.nav import NavGroup
from django_simple_nav.nav import NavItem


class ExampleListNav(Nav):
    template_name = "navs/example_list.html"
    items = [
        NavItem(title="Basic", url="/basic/"),
        NavItem(title="Permissions", url="/permissions/"),
        NavItem(title="Extra Context", url="/extra-context/"),
        NavItem(title="Nested `Nav`", url="/nested/"),
        NavGroup(
            title="Frameworks",
            items=[
                NavItem(title="Tailwind CSS", url="/tailwind/"),
                NavItem(title="Bootstrap 4", url="/bootstrap4/"),
                NavItem(title="Bootstrap 5", url="/bootstrap5/"),
                NavItem(title="Pico CSS", url="/picocss/"),
            ],
        ),
    ]


class BasicNav(Nav):
    template_name = "navs/basic.html"
    items = [
        NavItem(title="Home", url="/basic/"),
        NavItem(title="Link", url="#"),
        NavGroup(
            title="Group",
            items=[
                NavItem(title="Link", url="#"),
                NavItem(title="Another link", url="#"),
            ],
        ),
        NavGroup(
            title="Group with Link",
            url="#",
            items=[
                NavItem(title="Link", url="#"),
                NavItem(title="Another link", url="#"),
            ],
        ),
    ]


class PermissionsNav(Nav):
    template_name = "navs/basic.html"
    items = [
        NavItem(title="Everyone can see this link", url="#"),
        NavItem(
            title="You are authenticated",
            url="#",
            permissions=["is_authenticated"],
        ),
        NavItem(
            title="You are a staff member",
            url="#",
            permissions=["is_staff"],
        ),
        NavItem(
            title="You are a superuser",
            url="#",
            permissions=["is_superuser"],
        ),
        NavItem(
            title="You have the `demo_permission`",
            url="#",
            permissions=["demo_permission"],
        ),
    ]


class SetPermissionsNav(Nav):
    template_name = "navs/basic.html"
    items = [
        NavItem(title="AnonymousUser", url="/permissions/"),
        NavItem(
            title="`is_authenticated`",
            url="/permissions/?permission=is_authenticated",
        ),
        NavItem(
            title="`is_staff`",
            url="/permissions/?permission=is_staff",
        ),
        NavItem(
            title="`is_superuser`",
            url="/permissions/?permission=is_superuser",
        ),
        NavItem(
            title="Permission `demo_permission`",
            url="/permissions/?permission=demo_permission",
        ),
    ]


class ExtraContextNav(Nav):
    template_name = "navs/extra_context.html"
    items = [
        NavItem(title="Normal Link", url="#"),
        NavItem(title="Has Extra Context", url="#", extra_context={"foo": "bar"}),
    ]


class NestedNav(BasicNav):
    template_name = "navs/nested.html"


class TailwindMainNav(Nav):
    template_name = "navs/tailwind_main.html"
    items = [
        NavItem(title="Dashboard", url="/tailwind/"),
        NavItem(title="Team", url="#"),
        NavItem(title="Projects", url="#"),
        NavItem(title="Calendar", url="#"),
    ]


class TailwindProfileNav(Nav):
    template_name = "navs/tailwind_profile.html"
    items = [
        NavItem(title="Your Profile", url="#"),
        NavItem(title="Settings", url="#"),
        NavItem(title="Sign out", url="#"),
    ]


class Bootstrap4Nav(Nav):
    template_name = "navs/bootstrap4.html"
    items = [
        NavItem(title="Home", url="/bootstrap4/"),
        NavItem(title="Link", url="#"),
        NavItem(title="Disabled", url="#", extra_context={"disabled": True}),
        NavGroup(
            title="Dropdown",
            items=[
                NavItem(title="Action", url="#"),
                NavItem(title="Another action", url="#"),
                NavItem(title="Something else here", url="#"),
            ],
        ),
    ]


class Bootstrap5Nav(Bootstrap4Nav):
    template_name = "navs/bootstrap5.html"


class PicoCSSNav(Nav):
    template_name = "navs/picocss.html"
    items = [
        NavItem(title="Services", url="/picocss/"),
        NavGroup(
            title="Account",
            items=[
                NavItem(title="Profile", url="#"),
                NavItem(title="Settings", url="#"),
                NavItem(title="Security", url="#"),
                NavItem(title="Logout", url="#"),
            ],
        ),
    ]



from __future__ import annotations

DEFAULT_SETTINGS = {
    "ALLOWED_HOSTS": ["*"],
    "DEBUG": False,
    "CACHES": {
        "default": {
            "BACKEND": "django.core.cache.backends.dummy.DummyCache",
        }
    },
    "DATABASES": {
        "default": {
            "ENGINE": "django.db.backends.sqlite3",
            "NAME": ":memory:",
        }
    },
    "EMAIL_BACKEND": "django.core.mail.backends.locmem.EmailBackend",
    "LOGGING_CONFIG": None,
    "PASSWORD_HASHERS": [
        "django.contrib.auth.hashers.MD5PasswordHasher",
    ],
    "SECRET_KEY": "not-a-secret",
}



from __future__ import annotations

from dataclasses import dataclass
from dataclasses import field
from typing import Any

from django.core.exceptions import ImproperlyConfigured
from django.http import HttpRequest
from django.template.loader import render_to_string
from django.urls import reverse
from django.urls.exceptions import NoReverseMatch
from django.utils.safestring import mark_safe

from django_simple_nav.permissions import check_item_permissions


@dataclass(frozen=True)
class Nav:
    template_name: str | None = field(init=False, default=None)
    items: list[NavGroup | NavItem] | None = field(init=False, default=None)

    def get_template_name(self) -> str:
        if self.template_name is not None:
            return self.template_name

        msg = f"{self.__class__!r} must define 'template_name' or override 'get_template_name()'"
        raise ImproperlyConfigured(msg % self.__class__.__name__)

    def get_items(self, request: HttpRequest) -> list[RenderedNavItem]:
        if self.items is not None:
            return [
                RenderedNavItem(item, request)
                for item in self.items
                if check_item_permissions(item, request)
            ]

        msg = f"{self.__class__!r} must define 'items' or override 'get_items()'"
        raise ImproperlyConfigured(msg)

    def get_context_data(self, request: HttpRequest) -> dict[str, Any]:
        return {
            "items": self.get_items(request),
            "request": request,
        }

    def render(self, request: HttpRequest, template_name: str | None = None) -> str:
        context = self.get_context_data(request)
        return render_to_string(
            template_name=template_name or self.get_template_name(),
            context=context,
            request=request,
        )


@dataclass(frozen=True)
class NavGroup:
    title: str
    items: list[NavGroup | NavItem]
    url: str | None = None
    permissions: list[str] = field(default_factory=list)
    extra_context: dict[str, Any] = field(default_factory=dict)


@dataclass(frozen=True)
class NavItem:
    title: str
    url: str
    permissions: list[str] = field(default_factory=list)
    extra_context: dict[str, Any] = field(default_factory=dict)


@dataclass(frozen=True)
class RenderedNavItem:
    item: NavItem | NavGroup
    request: HttpRequest

    def __getattr__(self, name: str) -> Any:
        if name == "extra_context":
            return self.item.extra_context
        elif hasattr(self.item, name):
            return getattr(self.item, name)
        else:
            try:
                return self.item.extra_context[name]
            except KeyError as err:
                msg = f"{self.item!r} object has no attribute {name!r}"
                raise AttributeError(msg) from err

    @property
    def title(self) -> str:
        return mark_safe(self.item.title)

    @property
    def items(self) -> list[RenderedNavItem] | None:
        if not isinstance(self.item, NavGroup):
            return None
        return [RenderedNavItem(item, self.request) for item in self.item.items]

    @property
    def url(self) -> str | None:
        if not self.item.url:
            return None
        try:
            url = reverse(self.item.url)
        except NoReverseMatch:
            url = self.item.url
        return url

    @property
    def active(self) -> bool:
        if not self.url:
            return False
        return (
            self.request.path.startswith(self.url)
            and self.url != "/"
            or self.request.path == self.url
        )



from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from django.conf import settings

DJANGO_SIMPLE_NAV_SETTINGS_NAME = "DJANGO_SIMPLE_NAV"


@dataclass(frozen=True)
class AppSettings:
    def __getattribute__(self, __name: str) -> Any:
        user_settings = getattr(settings, DJANGO_SIMPLE_NAV_SETTINGS_NAME, {})
        return user_settings.get(__name, super().__getattribute__(__name))


app_settings = AppSettings()



from __future__ import annotations

from django_simple_nav.nav import Nav
from django_simple_nav.nav import NavGroup
from django_simple_nav.nav import NavItem


class DummyNav(Nav):
    template_name = "tests/dummy_nav.html"
    items = [
        NavItem(title="Relative URL", url="/relative-url"),
        NavItem(title="Absolute URL", url="https://example.com/absolute-url"),
        NavItem(title="Named URL", url="fake-view"),
        NavGroup(
            title="Group",
            url="/group",
            items=[
                NavItem(title="Relative URL", url="/relative-url"),
                NavItem(title="Absolute URL", url="https://example.com/absolute-url"),
                NavItem(title="Named URL", url="fake-view"),
            ],
        ),
        NavItem(
            title="is_authenticated Item", url="#", permissions=["is_authenticated"]
        ),
        NavItem(title="is_staff Item", url="#", permissions=["is_staff"]),
        NavItem(title="is_superuser Item", url="#", permissions=["is_superuser"]),
        NavItem(
            title="tests.dummy_perm Item", url="#", permissions=["tests.dummy_perm"]
        ),
        NavGroup(
            title="is_authenticated Group",
            permissions=["is_authenticated"],
            items=[NavItem(title="Test Item", url="#")],
        ),
        NavGroup(
            title="is_staff Group",
            permissions=["is_staff"],
            items=[NavItem(title="Test Item", url="#")],
        ),
        NavGroup(
            title="is_superuser Group",
            permissions=["is_superuser"],
            items=[NavItem(title="Test Item", url="#")],
        ),
        NavGroup(
            title="tests.dummy_perm Group",
            permissions=["tests.dummy_perm"],
            items=[NavItem(title="Test Item", url="#")],
        ),
    ]



from __future__ import annotations

import os
import secrets
import sys
from pathlib import Path

from django.conf import settings
from django.core.wsgi import get_wsgi_application
from django.shortcuts import render
from django.urls import path

BASE_DIR = Path(__file__).parent

settings.configure(
    ALLOWED_HOSTS="*",
    DATABASES={
        "default": {
            "ENGINE": "django.db.backends.sqlite3",
            "NAME": BASE_DIR / "db.sqlite3",
        },
    },
    INSTALLED_APPS=[
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django_simple_nav",
    ],
    LOGGING={
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": {
            "verbose": {
                "format": "%(asctime)s %(name)-12s %(levelname)-8s %(message)s",
            },
        },
        "handlers": {
            "stdout": {
                "class": "logging.StreamHandler",
                "stream": sys.stdout,
                "formatter": "verbose",
            },
        },
        "loggers": {
            "django": {
                "handlers": ["stdout"],
                "level": os.getenv("DJANGO_LOG_LEVEL", "INFO"),
            },
        },
    },
    MIDDLEWARE=[
        "django.contrib.sessions.middleware.SessionMiddleware",
        "django.contrib.auth.middleware.AuthenticationMiddleware",
    ],
    ROOT_URLCONF=__name__,
    SECRET_KEY=secrets.token_hex(32),
    TEMPLATES=[
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "DIRS": [BASE_DIR / "templates"],
            "OPTIONS": {
                "context_processors": [
                    "django.template.context_processors.request",
                ],
            },
        }
    ],
)


def demo(request, template_name):
    return render(request, template_name)


def permissions(request):
    perm = request.GET.get("permission", None)
    if perm:
        request.user = type(
            "User", (), {"is_authenticated": True, "has_perm": lambda p: p == perm}
        )
        if perm in ["is_staff", "is_superuser"]:
            setattr(request.user, perm, True)
    return demo(request, "permissions.html")


urlpatterns = [
    path("", demo, {"template_name": "base.html"}),
    path("basic/", demo, {"template_name": "basic.html"}),
    path("permissions/", permissions),
    path("extra-context/", demo, {"template_name": "extra_context.html"}),
    path("nested/", demo, {"template_name": "nested.html"}),
    path("tailwind/", demo, {"template_name": "tailwind.html"}),
    path("bootstrap4/", demo, {"template_name": "bootstrap4.html"}),
    path("bootstrap5/", demo, {"template_name": "bootstrap5.html"}),
    path("picocss/", demo, {"template_name": "picocss.html"}),
]

app = get_wsgi_application()


if __name__ == "__main__":
    from django.contrib.auth.models import Permission
    from django.core.management import call_command

    call_command("migrate")
    Permission.objects.update_or_create(
        codename="demo_permission",
        name="Demo Permission",
        content_type_id=1,
    )
    call_command("runserver")



from __future__ import annotations

from django import template
from django.utils.module_loading import import_string

register = template.Library()


@register.tag(name="django_simple_nav")
def do_django_simple_nav(parser, token):
    try:
        _, args = token.contents.split(None, 1)
    except ValueError as err:
        raise template.TemplateSyntaxError(
            f"{token.contents.split()[0]} tag requires arguments"
        ) from err

    args = args.split()
    nav = args[0]
    template_name = args[1] if len(args) > 1 else None

    return DjangoSimpleNavNode(nav, template_name)


class DjangoSimpleNavNode(template.Node):
    def __init__(self, nav, template_name):
        self.nav = template.Variable(nav)
        self.template_name = template.Variable(template_name) if template_name else None

    def render(self, context):
        try:
            nav = self.nav.resolve(context)
            template_name = (
                self.template_name.resolve(context) if self.template_name else None
            )
        except template.VariableDoesNotExist as err:
            raise template.TemplateSyntaxError(
                f"Variable does not exist: {err}"
            ) from err

        if isinstance(nav, str):
            try:
                nav_instance = import_string(nav)()
            except ImportError as err:
                raise template.TemplateSyntaxError(f"Failed to import: {nav}") from err
        else:
            nav_instance = nav

        if not hasattr(nav_instance, "render"):
            raise template.TemplateSyntaxError(
                "The object does not have a 'render' method."
            )

        return nav_instance.render(context["request"], template_name)



from __future__ import annotations

import os
from pathlib import Path

import nox

nox.options.default_venv_backend = "uv|virtualenv"
nox.options.reuse_existing_virtualenvs = True

PY38 = "3.8"
PY39 = "3.9"
PY310 = "3.10"
PY311 = "3.11"
PY312 = "3.12"
PY_VERSIONS = [PY38, PY39, PY310, PY311, PY312]
PY_DEFAULT = PY_VERSIONS[0]
PY_LATEST = PY_VERSIONS[-1]

DJ32 = "3.2"
DJ42 = "4.2"
DJ50 = "5.0"
DJMAIN = "main"
DJMAIN_MIN_PY = PY310
DJ_VERSIONS = [DJ32, DJ42, DJ50, DJMAIN]
DJ_LTS = [DJ32, DJ42]
DJ_DEFAULT = DJ_LTS[0]
DJ_LATEST = DJ_VERSIONS[-2]


def version(ver: str) -> tuple[int, ...]:
    """Convert a string version to a tuple of ints, e.g. "3.10" -> (3, 10)"""
    return tuple(map(int, ver.split(".")))


def should_skip(python: str, django: str) -> bool:
    """Return True if the test should be skipped"""

    if django == DJMAIN and version(python) < version(DJMAIN_MIN_PY):
        # Django main requires Python 3.10+
        return True

    if django == DJ32 and version(python) >= version(PY312):
        # Django 3.2 requires Python < 3.12
        return True

    if django == DJ50 and version(python) < version(PY310):
        # Django 5.0 requires Python 3.10+
        return True

    return False


@nox.session
def test(session):
    session.notify(f"tests(python='{PY_DEFAULT}', django='{DJ_DEFAULT}')")


@nox.session
@nox.parametrize(
    "python,django",
    [
        (python, django)
        for python in PY_VERSIONS
        for django in DJ_VERSIONS
        if not should_skip(python, django)
    ],
)
def tests(session, django):
    session.install("django-simple-nav[dev] @ .")

    if django == DJMAIN:
        session.install(
            "django @ https://github.com/django/django/archive/refs/heads/main.zip"
        )
    else:
        session.install(f"django=={django}")

    if session.posargs:
        session.run("python", "-m", "pytest", *session.posargs)
    else:
        session.run("python", "-m", "pytest")


@nox.session
def coverage(session):
    session.install("django-simple-nav[dev] @ .")
    session.run("python", "-m", "pytest", "--cov=django_simple_nav")

    try:
        summary = os.environ["GITHUB_STEP_SUMMARY"]
        with Path(summary).open("a") as output_buffer:
            output_buffer.write("")
            output_buffer.write("### Coverage\n\n")
            output_buffer.flush()
            session.run(
                "python",
                "-m",
                "coverage",
                "report",
                "--skip-covered",
                "--skip-empty",
                "--format=markdown",
                stdout=output_buffer,
            )
    except KeyError:
        session.run(
            "python", "-m", "coverage", "html", "--skip-covered", "--skip-empty"
        )

    session.run("python", "-m", "coverage", "report")


@nox.session
def lint(session):
    session.install("django-simple-nav[lint] @ .")
    session.run("python", "-m", "pre_commit", "run", "--all-files")


@nox.session
def mypy(session):
    session.install("django-simple-nav[dev] @ .")
    session.run("python", "-m", "mypy", ".")


@nox.session
def demo(session):
    addrport = session.posargs[0] if session.posargs else "localhost:8000"

    session.install("django-simple-nav[dev] @ .")
    session.run("python", "example/demo.py", "runserver", addrport)



from __future__ import annotations

import pytest
from django.conf import settings

from django_simple_nav.conf import DJANGO_SIMPLE_NAV_SETTINGS_NAME
from django_simple_nav.conf import app_settings


def test_default_settings():
    user_settings = getattr(settings, DJANGO_SIMPLE_NAV_SETTINGS_NAME, {})

    assert user_settings == {}


def test_app_settings():
    # temporary until app actually has settings
    with pytest.raises(AttributeError):
        app_settings.FOO  # noqa: B018
